<?php

  namespace Drupal\dxpr_theme_helper\Plugin\Block;

  use Drupal\Core\Access\AccessResult;
  use Drupal\Core\Block\BlockBase;
  use Drupal\Core\Entity\EntityTypeManagerInterface;
  use Drupal\Core\Session\AccountInterface;
  use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
  use Symfony\Component\DependencyInjection\ContainerInterface;
  use Drupal\Core\Entity\EntityFormBuilderInterface;
  use Drupal\Core\Config\ConfigFactoryInterface;

  /**
   * Provides a block for the user registration form.
   *
   * @Block(
   *   id = "dxpr_theme_helper_user_register",
   *   admin_label = @Translation("User registration form"),
   *   category = @Translation("Forms")
   * )
   *
   * Note that we set module to contact so that blocks will be disabled correctly
   * when the module is disabled.
   */
  class UserRegisterBlock extends BlockBase implements ContainerFactoryPluginInterface {

    /**
     * The entity manager.
     *
     * @var \Drupal\Core\Entity\EntityTypeManagerInterface
     */
    protected $entityTypeManager;

    /**
     * The entity form builder.
     *
     * @var \Drupal\Core\Entity\EntityFormBuilderInterface
     */
    protected $entityFormBuilder;

    /**
     * The config factory.
     *
     * @var \Drupal\Core\Config\ConfigFactoryInterface
     */
    protected $configFactory;

    /**
     * Constructs a new UserRegisterBlock plugin.
     *
     * @param array $configuration
     *   A configuration array containing information about the plugin instance.
     * @param string $plugin_id
     *   The plugin_id for the plugin instance.
     * @param mixed $plugin_definition
     *   The plugin implementation definition.
     * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entityTypeManager
     *   The entity manager.
     * @param \Drupal\Core\Entity\EntityFormBuilderInterface $entityFormBuilder
     *   The entity form builder.
     * @param \Drupal\Core\Config\ConfigFactoryInterface $configFactory
     *   The config factory.
     */
    public function __construct(array $configuration, $plugin_id, $plugin_definition, EntityTypeManagerInterface $entityTypeManager, EntityFormBuilderInterface $entityFormBuilder, ConfigFactoryInterface $configFactory) {
      parent::__construct($configuration, $plugin_id, $plugin_definition);
      $this->entityTypeManager = $entityTypeManager;
      $this->entityFormBuilder = $entityFormBuilder;
      $this->configFactory = $configFactory;
    }

    /**
     * {@inheritdoc}
     */
    public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
      return new static(
        $configuration,
        $plugin_id,
        $plugin_definition,
        $container->get('entity_type.manager'),
        $container->get('entity.form_builder'),
        $container->get('config.factory')
      );
    }

    /**
     * Implements \Drupal\block\BlockBase::build().
     */
    public function build() {
      $build = [];

      $account = $this->entityTypeManager->getStorage('user')->create([]);
      $build['form'] = $this->entityFormBuilder->getForm($account, 'register');

      return $build;
    }

    /**
     * {@inheritdoc}
     */
    public function blockAccess(AccountInterface $account) {
      $user_settings = $this->configFactory->get('user.settings');
      return AccessResult::allowedIf($account->isAnonymous() && ($user_settings->get('register') != \Drupal\user\UserInterface::REGISTER_ADMINISTRATORS_ONLY))
        ->addCacheContexts(['user.roles'])
        ->addCacheTags($user_settings->getCacheTags());
    }

  }
